/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

//-----------------------------------------------------------------------------
// Object: make an optimized output buffer (strcat like) function with reallocation if needed
//-----------------------------------------------------------------------------

#pragma once

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

class COutputString
{
protected:
    TCHAR* String;  // pointer to string
    SIZE_T Size;    // _tcslen like : size without \0
    SIZE_T MaxSize; // max size including \0
public:
    COutputString(SIZE_T InitialSize=1024)
    {
        this->MaxSize=InitialSize;
        this->String=new TCHAR[this->MaxSize];
        this->Clear();
    }
    ~COutputString(void)
    {
        delete[] this->String;
    }
    
    void IncreaseCapacity(SIZE_T NewSize);

    void Append(TCHAR CharToAdd);
    void Append(TCHAR* StringToAdd,SIZE_T StringToAddSize);
    FORCEINLINE void Append(TCHAR* StringToAdd){return this->Append(StringToAdd,_tcslen(StringToAdd));}
    void AppendW(WCHAR CharToAdd);
    void AppendW(WCHAR* UnicodeStringToAdd,SIZE_T StringToAddSize);
    FORCEINLINE void AppendW(WCHAR* StringToAdd){return this->AppendW(StringToAdd,wcslen(StringToAdd));}
    void AppendA(CHAR CharToAdd);
    void AppendA(CHAR* AsciiStringToAdd,SIZE_T StringToAddSize);
    FORCEINLINE void AppendA(CHAR* StringToAdd){return this->AppendA(StringToAdd,strlen(StringToAdd));}

    FORCEINLINE void Prepend(TCHAR* StringToAdd){return this->Prepend(StringToAdd,_tcslen(StringToAdd));}
    void Prepend(TCHAR* StringToAdd,SIZE_T StringToAddSize);

    FORCEINLINE TCHAR* GetString(){return this->String;}
    FORCEINLINE SIZE_T GetLength(){return this->Size;}
    FORCEINLINE SIZE_T GetMaxSize(){return this->MaxSize;}
    FORCEINLINE void Clear(){*this->String=0;this->Size=0;}
    int Snprintf(SIZE_T MaxCount,const TCHAR* Format,...);
    int AppendSnprintf(SIZE_T MaxCount,const TCHAR* Format,...);
    int AppendVsnprintf(SIZE_T MaxCount,const TCHAR* Format,va_list args);
    // Ellipse : return TRUE is Ellipse has been done, FALSE is current size is less than MaxWantedNbChars
    BOOL Ellipse(SIZE_T MaxWantedNbChars);

    // if after GetString, user change string content manually (like end string after _tcspbrk checking)
    // he HAS TO call this function to update class members, else next Append will fail
    FORCEINLINE void UpdateAfterExternalContentChange(){this->Size=_tcslen(this->String);}
};

